
#THE FOLLOWING CODE REQUIRES TO HAVE THE PACKAGES OPENPYXL, SCIPY, SEABORN, MATH, AND PANDAS

#LOADING OF PACKAGES AND MODULES
import openpyxl
from scipy import stats
import seaborn as sns
from scipy.stats import rankdata
import math
import pandas as pd


#DEFINE THE KEGG METABOLITES CONTAINED IN CLUSTERS
#STEP NR 1 - LOAD THE INPUT DATA. TABULAR (.XLSX) DATA WITH THE FOLLOWING COLUMNS FOR EACH ELEMENT (WHEN POSSIBLE):
#COLUMNS (WITH HEADERS): 
#CLUSTER ID
#KEGG COMPOUND ID OF ASSOCIATED NODES
#KEGG COMPOUND ID OF MAPPED NODE
#METABOLITE NAME
#SMBL MODEL IDENTIFIER
reference_workbook = input('Please provide path and filename of reference workbook and press ENTER: ') 
wb = openpyxl.load_workbook(str(reference_workbook))
reference_sheet = input('Please provide name of working sheet and press ENTER: ')
sh = wb[str(reference_sheet)]

#INDICATE COLUMNS
col_cluster_id = input('Please indicate column letter of cluster id and press ENTER: ')
col_KEGG_univ = input('Please indicate column letter of associated KEGG compound IDs and press ENTER: ')
col_KEGG_mapped = input('Please indicate column letter of mapped KEGG compound IDs and press ENTER: ')
col_met_name = input('Please indicate column letter of mapped KEGG compound IDs and press ENTER: ')
col_met_id = input('Please indicate column letter of model identifier and press ENTER: ')
#OBTAIN THE NUMBER OF CLUSTERS CONTAINED IN THE FILE
clusternr_list = []
for i in range (2, (sh.max_row+1)):
	cluster_id = sh[str(col_cluster_id)+str(i)].value
	clusternr_list.append(cluster_id)
cluster_number = (max(clusternr_list))
print ('TOTAL NUMBER OF CLUSTERS: '+str(cluster_number))

#OBTAIN THE METABOLITE UNIVERSE (ALL KEGG COMPOUND IDENTIFIERS MAPPED IN THE LIST)
mapped_universe = []
for k in range (2, (sh.max_row+1)):
	metabolite_univ = sh[str(col_KEGG_univ)+str(k)].value
	if metabolite_univ != None:
		mapped_universe.append(metabolite_univ)
mapped_universe = list(dict.fromkeys(mapped_universe))
mapped_univ_nr = len(mapped_universe)
print ('TOTAL NUMBER OF METABOLITES IN UNIVERSE: '+str(mapped_univ_nr))


#OBTAIN THE METABOLITE LIST QUERY (DEDUPLICATED)
mapped_mets = []
for k in range (2, (sh.max_row+1)):
	metabolite = sh[str(col_KEGG_mapped)+str(k)].value
	if metabolite != None:
		mapped_mets.append(metabolite)
mapped_mets = list(dict.fromkeys(mapped_mets))
mapped_nr = len(mapped_mets)
print ('TOTAL NUMBER OF DEDUPLICATED METABOLITES IN LIST QUERY: '+str(mapped_nr))


#INITIAL FORMATTING OF OUTPUT FILE
wb_ORA = openpyxl.Workbook()
sh_ORA = wb_ORA['Sheet']
sh_ORA['A1'] = 'Cluster name'
sh_ORA['B1'] = 'Cluster small molecules with KEGG ID'
sh_ORA['C1'] = 'Mapped small molecules'
sh_ORA['D1'] = 'Total'
sh_ORA['E1'] = 'Hits'
sh_ORA['F1'] = 'Expect'
sh_ORA['G1'] = 'Enrichment ratio'
sh_ORA['H1'] = 'pvalue'
sh_ORA['I1'] = '-log(P)'



#GET KEGG COMPOUND ID LISTS PRESENT IN NRMETS FROM EACH CLUSTER, BOTH UNIVERSE AND MAPPED
for i in range(1, cluster_number+1):
	cluster_kegg = []
	cluster_kegg_present = []
	cluster_name = []
	cluster_name_present = []
	for j in range(2, (sh.max_row+1)):
		cluster_id2 = sh[str(col_cluster_id)+str(j)].value
		if cluster_id2 == i:
			KEGG_id = sh[str(col_KEGG_univ)+str(j)].value
			KEGG_id_present = sh[str(col_KEGG_mapped)+str(j)].value
			metabolite_name = sh[str(col_met_name)+str(j)].value
			SMBL_id = sh[str(col_met_id)+str(j)].value
			if KEGG_id != None:
				cluster_kegg.append(KEGG_id)
				cluster_name.append(str(metabolite_name)+'['+str(SMBL_id)+']')
				if KEGG_id_present != None:
					cluster_kegg_present.append(KEGG_id_present)
					cluster_name_present.append(str(metabolite_name)+'['+str(SMBL_id)+']')
	nr_mets_cluster = len(cluster_kegg)
	nr_mets_cluster_exp = len(cluster_kegg_present)
	#CALCULATION OF EXPECTED METABOLITE NUMBER AND ENRICHMENT
	expected = ((mapped_nr*nr_mets_cluster)/mapped_univ_nr)
	if expected != 0:
		enrich_ratio = round((nr_mets_cluster_exp/expected), 3)
	else:
		enrich_ratio = 0
	#PERFORMANCE OF HYPERGEOMETRIC TEST
	pval = 1 - stats.hypergeom.cdf((nr_mets_cluster_exp-1), mapped_univ_nr, mapped_nr, nr_mets_cluster)
	logpval = math.log10(pval)*-1
	sh_ORA['A'+str(i+1)] = i
	sh_ORA['B'+str(i+1)] = str(cluster_name)
	sh_ORA['C'+str(i+1)] = str(cluster_name_present)
	sh_ORA['D'+str(i+1)] = nr_mets_cluster
	sh_ORA['E'+str(i+1)] = nr_mets_cluster_exp
	sh_ORA['F'+str(i+1)] = expected
	sh_ORA['G'+str(i+1)] = enrich_ratio
	sh_ORA['H'+str(i+1)] = pval
	sh_ORA['I'+str(i+1)] = logpval
#PERFORMANCE OF BENHAMINI-HOCHBERG CORRECTION
data = sh_ORA.values
column_headers = next(data)[0:]
df = pd.DataFrame(data, columns=column_headers)
print (df)
#df = pd.read_excel(str(out1)+'.xlsx')
df.sort_values('pvalue')
df.reset_index(drop = True)
df['BH'] = df.pvalue * len(df) / (df.index + 1)
df.loc[df['BH'] > 1, 'BH'] = 1
print (df)
#GENERATION OF FILE WITH ENRICHMENT RESULTS
out2 = input('Please type the output file name of results without multiple comparisons correction and press ENTER :')
df.to_csv(str(out2)+'.csv')
